#include <v9990.h>
#include <stdio.h>
#include <stdint.h>
#include "Fireflies.h"

#define SID_BASE        0x3df800
#define READ_SID(x)	(*((volatile char *) SID_BASE + (x) ))
#define WRITE_SID(x, y) (*((volatile uint8_t *) SID_BASE + (x) ) = (y) )
#define SID2_BASE       0x3df900
#define WRITE_SID2(x, y)        (*((volatile uint8_t *) SID2_BASE + (x) ) = (y) )

#define IDE_BASE  0x003dfd00
#define IDE_DATA  0x00
#define IDE_FEATURES 0x02
#define IDE_SEC_COUNT 0x04
#define IDE_LBA_LOW 0x06
#define IDE_LBA_MID 0x08
#define IDE_LBA_HIGH  0x0a
#define IDE_DRV_HEAD  0x0c
#define IDE_COMMAND 0x0e
#define IDE_CONTROL 0x10c
#define READ_IDE(x) (*((volatile char *) IDE_BASE + (x) ))
#define WRITE_IDE(x, y) (*((volatile char *) IDE_BASE + (x) ) = (y) ); (*((volatile char *) IDE_BASE + (x) + 1 ) = 0x20 )
#define WRITE_IDE_DATA(x, y)  (*((volatile char *) IDE_BASE + (x) ) = (y) )


static uint8_t sid[] = {0x02,0x00,0x08,0x0a,0x0c,0x06,0x04,0x10,0x0e,0x16,0x18,0x1a,0x14,0x12,0x1e,0x1c,0x24,0x26,0x28,0x22,0x20,0x2c,0x20,0x11};
uint16_t sound_frame, sound_pos;
uint8_t volume;
#define VOLUME 48

void ISR1(void) __attribute__((interrupt));
void ISR1(void) {
  static uint32_t idx;
  static uint8_t cnt;
  if (sound_frame < sound_data_frames) {
    idx=sound_data[sound_pos++]; // read number of register/value pairs for this frame
    for (cnt=0;cnt<idx;cnt++) {
        WRITE_SID(sound_data[sound_pos++]<<1,sound_data[sound_pos++]); // read/write register and value
    }
    sound_frame++;
  } else {
    sound_frame=0;
    sound_pos=0;
  }
  WRITE_VDP(VDP_IFP,0x07);	// IRQ loeschen
}


void sound_init() {
  uint8_t i;
  volume=15;
  for (i=0;i<50;i++) {
    WRITE_SID(i,0x00);
    WRITE_SID2(i,0x00);
  }
  WRITE_SID(VOLUME,volume);	
  WRITE_SID2(VOLUME,volume);	
}

void main() {
  WRITE_IDE(IDE_DRV_HEAD, 0x10); /* No IDE Interrupts */
  WRITE_IDE(IDE_CONTROL, 0x06); /* No IDE Interrupts */
  WRITE_IDE(IDE_CONTROL, 0x02); /* No IDE Interrupts */
  READ_IDE(IDE_COMMAND);
  WRITE_IDE(IDE_DRV_HEAD, 0x00); /* No IDE Interrupts */
  WRITE_IDE(IDE_CONTROL, 0x06); /* No IDE Interrupts */
  WRITE_IDE(IDE_CONTROL, 0x02); /* No IDE Interrupts */
  READ_IDE(IDE_COMMAND);

  sound_frame = 0;
  sound_pos = 0;
  asm volatile ("move.w   #0x2700, %sr");	// Interrupts aus
  (*((long *)0x64) = (long)ISR1);
  VDPWriteReg(VDP_INT_ENABLE,VDP_INT_IEV);
  WRITE_VDP(VDP_IFP,0x07);	
  asm volatile ("move.w   #0x2000, %sr"); // Interrupts ein
  while(1);
}
